#!/usr/bin/python
# -*- coding: utf-8 -*-
############################################################################
#                                                                          #
#    Copyright (C) 2007 by Romain Henriet                                  #
#    henriet.romain@libertysurf.fr                                         #
#                                                                          #
#    This program is free software; you can redistribute it and/or modify  #
#    it under the terms of the GNU General Public License as published by  #
#    the Free Software Foundation; either version 2 of the License, or     #
#    (at your option) any later version.                                   #
#                                                                          #
#    This program is distributed in the hope that it will be useful,       #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#    GNU General Public License for more details.                          #
#                                                                          #
#    You should have received a copy of the GNU General Public License     #
#    along with this program; if not, write to the                         #
#    Free Software Foundation, Inc.,                                       #
#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
#                                                                          #
############################################################################

import os, karamba, sys
from xml.dom.minidom import parse
from HTMLParser import HTMLParser
from gettext import GNUTranslations
from StringIO import StringIO
from kdecore import *
from commands import getoutput
import locale, time, marshal, re


locale.setlocale(locale.LC_ALL, '')

notesGatherer = None
timerReloadTags = 10
currentPostIt = 0
myPostIts = []
title=None
titleText=None
smallNote = False
defaultColor = None
configOptions={}
skippedOnce = False
showTags = True
desiredTag = ''
basketBase = ''
myIconLoader = None
customTheme=False
titleType=None
refreshRate=None
cmpt_refresh=2

# Highlight
highlightOptions={}
highlightMenu=None
highlightIcon=None
hlCurrentNote=None
hlFilePath = os.environ['HOME']+'/.superkaramba/dektopbasket_highlights'

# Backround images
bgImg=0
bgImgFile = None
ttBarImg = None
ttBarImgFile = None

# Actions
moveRightButton=None
moveLeftButton=None
cfgButton=None
launchBasketButton=None
select_img_pid = None
select_title_pid = None
select_size_pid = None
select_color_pid = None
select_ttBar_pid = None
select_tag_pid = None
select_refresh_pid = None

# Dimensions
size = [0,0]
xborder = 20
yborder = 12
thumbnails = [0,0]

# Translation
_ = lambda(x): x

# HTML colors
colors = {'black':'000000000', 'white':'255255255', 'red':'255000000', 'blue':'000000255', 'yellow':'2552550', 'orange':'255165000', 'green':'000255000', 'purple':'160032240', 'maroon':'176048096', 'grey':'190190190', 'navy':'000000128', 'silver': '230230250'}

class MyParser(HTMLParser):
	
	def __init__(self):
		HTMLParser.__init__(self)
		self.reset()
	
	def handle_starttag(self, tag, attrs):
		if tag == 'body':
			self.start = self.getpos()

	def handle_endtag(self, tag):
		if tag == 'body':
			self.end=self.getpos()
			self.end = (self.end[0], self.end[1]+7)

	def getPositions(self):
		return self.start, self.end

	def reset(self):
		HTMLParser.reset(self)
		self.content = ''
		self.start = None
		self.end = None

class Note:

	def __init__(self, path, type):
		self.text = None
		self.basket = None
		self.otherTags = []
		self.path = path
		self.isHighlighted = 'None'
		self.type = type

	def getFormattedText(self):
		#print self.otherTags
		
		return '<font color=\"'+defaultColor+'\"> '+ self.text + '</font>'

class karambaNote:

	def __init__(self, widget, note, ypos):
		self.note = note
		self.rt = karamba.createRichText(widget, note.getFormattedText())
		dim_text = karamba.getRichTextSize(widget, self.rt)
		nb_tags = len(note.otherTags)
		self.icons = []
		S = dim_text[0]*dim_text[1] + nb_tags * 400
		k = S/((size[0]-2*xborder-18)*20)
		if k==0:
			k=1
		kk = int(nb_tags/k)+1
		
		karamba.setRichTextWidth(widget, self.rt, size[0]-kk*18-2*xborder-18)
		karamba.moveRichText(widget, self.rt, xborder+18, ypos)
		cmpt_v = 0
		xpos = size[0]- 16 - xborder
		for i in note.otherTags:
			image = karamba.createImage(widget, xpos, ypos + 18*cmpt_v +3 , i)
			karamba.resizeImage(widget, image, 16, 16)	# for custom iconsé
			if cmpt_v==k-1:
				cmpt_v = 0
				xpos = xpos - 18
			else:
				cmpt_v = cmpt_v+1
			#print i
			self.icons.append(image)
		self.highlight=None
		if note.isHighlighted:
			h = karamba.getRichTextSize(widget, self.rt)[1]
			self.highlight = karamba.createImage(widget, xborder, ypos+1, 'images/highlight/'+note.isHighlighted)
			karamba.resizeImage(widget, self.highlight, size[0]-2*xborder, h+1)
		self.hlButton = karamba.createImage(widget, xborder, ypos+3, highlightIcon)
		karamba.attachClickArea(widget, self.hlButton)

	def changeHighlight(self, widget, color):
		if self.note.isHighlighted==color:	#no change
			return
		if self.highlight:
			karamba.deleteImage(widget, self.highlight)
			self.highlight=None
		try:
			hlFile = open (hlFilePath, 'rb')
			highlights = marshal.load(hlFile)
			hlFile.close()
		except:
			highlights={}
		if color=='None':
			self.note.isHighlighted='None'
			del highlights[self.note.path]
		else:
			self.note.isHighlighted=color
			highlights[self.note.path]=color
			xy = karamba.getRichTextPos(widget, self.rt)
			self.highlight = karamba.createImage(widget, xborder, xy[1]+1, 'images/highlight/'+color)
			xy = karamba.getRichTextSize(widget, self.rt)
			karamba.resizeImage(widget, self.highlight, size[0]-2*xborder, xy[1]+1)
			karamba.attachClickArea(widget, self.hlButton)
		hlFile = open (hlFilePath, 'wb')
		marshal.dump(highlights, hlFile)
		hlFile.close()

	def move(self, widget, ypos):
		xy = karamba.getRichTextPos(widget, self.rt)
		diff_y = ypos-xy[1]
		karamba.moveRichText(widget, self.rt, xy[0], diff_y+xy[1])
		for i in self.icons:
			xy = karamba.getImagePos(widget, i)
			karamba.moveImage(widget, i, xy[0], diff_y+xy[1])
		if self.highlight:
			xy = karamba.getImagePos(widget, self.highlight)
			karamba.moveImage(widget, self.highlight, xy[0], diff_y+xy[1])
		xy = karamba.getImagePos(widget, self.hlButton)
		karamba.moveImage(widget, self.hlButton, xy[0], diff_y+xy[1])
			
	def delete(self, widget):
		for i in self.icons:
			karamba.deleteImage(widget, i)
		if self.highlight:
			karamba.deleteImage(widget, self.highlight)
			self.highlight=None
		karamba.deleteRichText(widget, self.rt)
		karamba.deleteImage(widget, self.hlButton)

	def show(self, widget, s):
		if s:
			karamba.showRichText(widget, self.rt)
			for i in self.icons:
				karamba.showImage(widget, i)
			if self.highlight:
				karamba.showImage(widget, self.highlight)
			karamba.showImage(widget, self.hlButton)
		else:
			karamba.hideRichText(widget, self.rt)
			for i in self.icons:
				karamba.hideImage(widget, i)
			if self.highlight:
				karamba.hideImage(widget, self.highlight)
			karamba.hideImage(widget, self.hlButton)


class PostIt:

	def __init__(self):
		self.notes = []

	def append(self, kn):
		self.notes.append(kn)

	def show(self, widget, s):
		for n in self.notes:
			n.show(widget, s)

	def delete(self, widget):
		for n in self.notes:
			n.delete(widget)

# Searches for baskets, parses their contents and 
# reads notes
class NotesGatherer:

	def __init__(self):
		self.base = basketBase
		self.lastTagsChange = None
		self.choosingTags = {}
		self.lastBasketListChange = None
		self.basketsList=[]
		self.basketsChanges=[]
		self.needUpdate = True
		self.desiredTag = desiredTag
		self.reloadTags()
		
	def main(self):
		#self.needUpdate = False
		self.notesList=[]
		self.myHTMLParser = MyParser()
		self.getBaskets()
		self.reloadTags()
		if self.desiredTag!=desiredTag:
			self.needUpdate=True
			self.desiredTag=desiredTag
		if not self.needUpdate:
			for i in range(0,len(self.basketsList)):
				d = os.stat(self.basketsList[i]).st_mtime
				if d!=self.basketsChanges[i]:
					self.needUpdate=True
					self.basketsChanges[i]=d
			if not self.needUpdate:
				return self.notesList
		self.getNotes()
		#print self.notesList
		self.readNotes()
		return self.notesList

	# gets baskets list
	def getBaskets(self):
		stat = os.stat(self.base+"baskets/baskets.xml")
		if self.lastBasketListChange==stat.st_mtime:
			print 'Basket list didn\'t change'
			return
		self.needUpdate = True
		self.basketsList=[]
		self.basketsChanges=[]
		self.lastBasketListChange = stat.st_mtime
		print 'Looking for baskets'
		doc = parse (self.base+"baskets/baskets.xml")
		baskets = doc.getElementsByTagName("basket")
		
		for b in baskets :
			name = b.attributes.get("folderName", None)
			prot = b.getElementsByTagName('protection')[0]
			if prot.attributes.get('type').value != '0': # basket is protected
				continue # skip it
			self.basketsList.append(self.base+'baskets/'+name.value)
			self.basketsChanges.append(0)
			print len(self.basketsList), 'baskets found'
		if len(self.basketsChanges)!=len(self.basketsList):
			print 'Error timestamps list not as long as baskets list'

	# extracts notes from the baskets
	def getNotes(self):
		print 'Looking for notes'
		for b in self.basketsList:
			print 'in',b
			try:
				doc = parse ( b + u'.basket')
			except:
				print "Error while parsing basket", b
			notes = doc.getElementsByTagName("note")

			for n in notes:
				typeN = n.attributes.get("type", None)
				if typeN == None:
					continue
				typeN = typeN.value
				
				if typeN != u'html' and typeN != u'image':# and typeN != u'link':	# only show text or link notes
					continue
				tag = n.getElementsByTagName("tags")
				if len(tag)==0:
					continue
				tags = tag[0].childNodes[0].nodeValue
				tags = tags.rsplit(';')
				if self.desiredTag not in tags:	# skip the note if there isn't the "desktop" tag
					continue
				print 'Removing tag', self.desiredTag
				tags.remove(self.desiredTag)
				ct = n.getElementsByTagName("content")[0]
				c =ct.childNodes[0]
				#switch according to note's type
				if typeN == u'html': # text
					newNote = Note(b+c.nodeValue, 'text')
				elif typeN == u'link':	# link
					linkText = ct.attributes.get("title")
					linkText = linkText.childNodes[0].nodeValue
					newNote = Note(None)
				else: # image
					newNote = Note(b+c.nodeValue, 'img')
					newNote.text = '<img src="'+b+c.nodeValue+'" height="54" />'
				if showTags:
					for t in tags:	# save the note's tags
						#print 'tag', t, self.tags.has_key(t)
						if self.tags.has_key(t):
							newNote.otherTags.append(self.tags[t])
				self.notesList.append(newNote)
			print len(self.notesList),"notes found"

	# reads notes contents
	def readNotes(self):
		try:
			hlFile = open (hlFilePath, 'rb')
			highlights = marshal.load(hlFile)
			hlFile.close()
		except:
			highlights = {}
		highlightsNew = {}
		print 'Reading notes content'
		text = ''
		for n in self.notesList:
			#print "Reading", n
			if n.path==None or not os.path.isfile(n.path) :
				continue
			if n.type == 'text':
				
				doc = open (n.path)
				data = doc.read()
				doc.close()
				doc = open (n.path)
				self.myHTMLParser.reset()
				self.myHTMLParser.feed(data)
				self.myHTMLParser.close()
				c = self.myHTMLParser.getPositions()
				aa = self.readPartOfFile(doc, c[0], c[1])
				n.text = aa
				try:
					n.text = unicode(aa, 'utf8')
				except:
					print "Error while decoding note(utf8)"
					try:
						n.text = unicode(aa, 'Latin1')
					except:
						print "Error while decoding note(Latin1)"
				doc.close()
			try:
				tmp = highlights[n.path]
				highlightsNew[n.path]=tmp
				n.isHighlighted=tmp
			except:
				pass
			#print n, "read"
		try:
			hlFile = open (hlFilePath, 'wb')
			marshal.dump(highlightsNew, hlFile)
			hlFile.close()
		except:
			pass
		print 'ok, all notes read'
			
	def readPartOfFile(self, doc, start, end):
		#print start, end
		for i in range(start[0]-1):
			doc.readline()
		text = ''
		if start[0] < end[0]-1:
			for i in range(start[0], end[0]):
				text = text + doc.readline()
		lastLine = doc.readline()
		lastLine = lastLine[:end[1]]
		text = text + lastLine
		return text[start[1]:]

	def reloadTags(self):
		if not os.path.isfile(self.base+ 'tags.xml'):
			print "Tags file not found !"
			self.tags={}
			return
		stat = os.stat(self.base+ 'tags.xml')
		if stat.st_mtime==self.lastTagsChange:
			print 'Tags didn\'t change'
			return
		self.needUpdate = True
		self.lastTagsChange=stat.st_mtime
		print 'Loading tags'
		self.tags = {}
		doc = parse (self.base+ 'tags.xml')
		#print 'Ok, tags file parsed'
		tags = doc.getElementsByTagName("state")
		#print "Found", len(tags), "tags"
		for t in tags:
			tagId = t.attributes.get('id').value
			name=[]
			try:
				name = t.getElementsByTagName("name")[0].childNodes
			except:
				pass
			if name==[]:
				name = tagId
			else:
				name = name[0].nodeValue
			self.choosingTags[name]=tagId
			#print name, '=====>', tagId
			icon = t.getElementsByTagName("emblem")
			if icon==[]:
				print "The tags file may be corrupted"
				continue
			icon = icon[0].childNodes
			if len(icon) != 0:
				iconName = icon[0].nodeValue
				#if icon[0].nodeValue.startswith('/'):
				self.tags[tagId]= str(myIconLoader.iconPath(iconName, KIcon.Small, False))

def init_language(widget):

	global _
	lang = karamba.userLanguages(widget)
	langFile = None

	for l in lang:
		fileContents = karamba.readThemeFile(widget, 'locales/'+l+'.mo')
		if fileContents.__len__() > 0:
			gt = GNUTranslations(StringIO(fileContents))
			_ = gt.ugettext
			break

def getBasketBase():
	##basketBase = os.environ['HOME']+'/.kde/share/apps/basket/'
	#a = getoutput('kde-config --localprefix')
	#basketBase = a + 'share/apps/basket/'
	#try:
		#cfgbase = getoutput('kde-config --path config').split(':')
	#except:
		#return basketBase
	#cfgfile =''
	#for i in cfgbase:
		#if os.path.isfile(i+'basketrc'):
			#cfgfile = i+'basketrc'
			#break
	#tmpBasketBase=''
	#if cfgfile: # time to get out the real path to baskets
		#try:
			#tmpBasketBase = re.search('(?<=dataFolder=).+',open(cfgfile).read()).group()
		#except:
			#pass
	#else:
		#return basketBase
	#try:
		#tmpBasketBase = tmpBasketBase.replace('$HOME',os.environ['HOME'])
		#tmpBasketBase = basketBase + '/'
		## now it should look like '/home/user/somelocation/basket/'
	#except:
		#pass
	#if tmpBasketBase=='/':
		#return basketBase
	#return tmpBasketBase
	a = getoutput('kde-config --localprefix')
	basketBase = a + 'share/apps/basket/'
	cfgfile= a +'share/config/basketrc'
	print 'Basket config file :', cfgfile
	if not os.path.isfile(cfgfile):
		return basketBase
	f = open(cfgfile)
	try:
		tmpBasketBase = re.search('(?<=dataFolder=).+',f.read()).group()
		f.close()
	except:
		f.close()
		return basketBase
	tmpBasketBase = tmpBasketBase.replace('$HOME',os.environ['HOME']) + '/'
	print 'Basket baskets folder :', tmpBasketBase
	if tmpBasketBase=='/':
		return basketBase
	return tmpBasketBase
	

def loadConfig(widget):
	global size, titleText, smallNote, bgImgFile, defaultColor, xborder, yborder, ttBarImgFile, ttBarImg
	global showTags, desiredTag, customTheme, titleType, refreshRate
	
	print "Loading config"
	smallNote = karamba.readConfigEntry(widget, "isReduced")=='True'
	showTags = karamba.readConfigEntry(widget, "showTags")!='False'
	karamba.writeConfigEntry(widget, "isReduced", str(smallNote))
	
	desiredTag = karamba.readConfigEntry(widget, "desiredTag")

	titleType = karamba.readConfigEntry(widget, "titleType")
	if not titleType:
		titleType='Text'
		karamba.writeConfigEntry(widget, "titleType", titleType)

	if titleType=='Date':
		titleText = ""
	else:
		titleText = karamba.readConfigEntry(widget, "title")
		if not titleText:
			titleText = 'DesktopBasket'
			karamba.writeConfigEntry(widget, "title", titleText)
	
	size[0] = karamba.readConfigEntry(widget, "widgetWidth")
	if not size[0]:
		size[0]=350

	refreshRate = karamba.readConfigEntry(widget, "refreshRate")
	if not refreshRate:
		refreshRate=30

	size[1] = karamba.readConfigEntry(widget, "noteHeight")
	if not size[1]:
		size[1]=200
		karamba.writeConfigEntry(widget, "noteHeight", str(size[1]))
	bgImgFile = karamba.readConfigEntry(widget, "bgImg")
	if not bgImgFile:
		bgImgFile = 'images/background.png'
		karamba.writeConfigEntry(widget, "bgImg", bgImgFile)
	ttBarImgFile = karamba.readConfigEntry(widget, "titleBar")
	if not ttBarImgFile:
		ttBarImgFile = 'images/TitleBar.png'
		karamba.writeConfigEntry(widget, "titleBar", ttBarImgFile)
	defaultColor = karamba.readConfigEntry(widget, "defaultColor")
	if not defaultColor:
		defaultColor = 'black'
		karamba.writeConfigEntry(widget, "defaultColor", defaultColor)
	else:
		try:
			titleText=unicode(titleText, 'utf8')
		except:
			try:
				titleText=unicode(titleText, 'Latin1')
			except:
				pass
	print "Config Loaded"
		
#this is called when your widget is initialized
def initWidget(widget):

	print "Starting init..."
	global _, notesGatherer, size, bgImg, moveLeftButton, moveRightButton
	global launchBasketButton, title, cfgButton, titleText, smallNote, bgImgFile
	global defaultColor, configMenu, configOptions, xborder, yborder, titleType
	global ttBarImgFile, ttBarImg, myPostIts, showTags, desiredTag, basketBase, myIconLoader
	global highlightOptions, highlightMenu, highlightIcon

	#aboutdata = KAboutData("DesktopBasket", "DesktopBasket", "1.2")
	#KCmdLineArgs.init(['desktopbasket.py'], aboutdata)
	myIconLoader = KGlobal.iconLoader()

	loadConfig(widget)
	basketBase = getBasketBase()
	init_language(widget)
	karamba.addMenuConfigOption(widget, 'showTags', _('Show tags'))
	karamba.setMenuConfigOption(widget, 'showTags', showTags)
	karamba.addMenuConfigOption(widget, 'titleDate', _('Show clock in title'))
	karamba.setMenuConfigOption(widget, 'titleDate', titleType=='Date')

	configMenu = karamba.createMenu(widget)
	configOptions['refresh'] = karamba.addMenuItem(widget, configMenu,_("Refresh"), 'reload')
	configOptions['refreshRate'] = karamba.addMenuItem(widget, configMenu,_("Change refresh rate"), 'reload')
	karamba.addMenuSeparator(widget, configMenu)
	configOptions['title'] = karamba.addMenuItem(widget, configMenu,_("Change title"), 'text')
	configOptions['color'] = karamba.addMenuItem(widget, configMenu,_("Change default text color"), 'colorize')
	configOptions['size'] = karamba.addMenuItem(widget, configMenu,_("Change note size"), 'crop')
	configOptions['selecttag'] = karamba.addMenuItem(widget, configMenu,_("Choose tag"), 'find')
	karamba.addMenuSeparator(widget, configMenu)
	configOptions['titleBar'] = karamba.addMenuItem(widget, configMenu,_("Change title bar"), 'frame_image')
	configOptions['background'] = karamba.addMenuItem(widget, configMenu,_("Change background"), 'frame_image')
	configOptions['resetTheme'] = karamba.addMenuItem(widget, configMenu,_("Reset theme"), 'remove')

	highlightMenu = karamba.createMenu(widget)
	themePath = karamba.getThemePath(widget)
	a= os.listdir(themePath+'/images/highlight')
	highlightOptions["None"] = karamba.addMenuItem(widget, highlightMenu, '', themePath+'/images/highlight/None')
	for i in a:
		if i.endswith('.png'):
			highlightOptions[i] = karamba.addMenuItem(widget, highlightMenu, '', themePath+'/images/highlight/'+i)

	notesGatherer = NotesGatherer()

	if not desiredTag:
		askForDesiredTag(widget)
	
	karamba.resizeWidget(widget, size[0], size[1])
	bgImg = karamba.createBackgroundImage(widget, 0, 20, bgImgFile)
	ttBarImg = karamba.createBackgroundImage(widget, 0, 0, ttBarImgFile)
	karamba.resizeImage(widget, bgImg, size[0],size[1]-20)
	karamba.resizeImage(widget, ttBarImg, size[0],20)
	moveLeftButton = karamba.createImage(widget, xborder, 2, str(myIconLoader.iconPath('1leftarrow', KIcon.Small, False)))
	moveRightButton = karamba.createImage(widget, 18+xborder, 2, str(myIconLoader.iconPath('1rightarrow', KIcon.Small, False)))
	launchBasketButton = karamba.createImage(widget, size[0]-16-xborder, 2, str(myIconLoader.iconPath('basket', KIcon.Small, False)))
	cfgButton = karamba.createImage(widget, size[0]-36-xborder, 2, str(myIconLoader.iconPath('configure', KIcon.Small, False)))
	title = karamba.createText(widget, xborder+40, -1, size[0]-80-2*xborder, 20, titleText)
	karamba.changeTextColor(widget, title, 0, 0, 0)
	karamba.setTextAlign(widget, title, 'CENTER')
	karamba.changeTextSize(widget, title, 14)
	karamba.attachClickArea(widget, moveLeftButton)
	karamba.attachClickArea(widget, moveRightButton)
	karamba.attachClickArea(widget, launchBasketButton)
	karamba.attachClickArea(widget, cfgButton)
	karamba.attachClickArea(widget, title)

	highlightIcon = str(myIconLoader.iconPath('bookmark', KIcon.Small, False))

	myPostIts = []
	tmpSmall = smallNote
	smallNote = False
	reloadNotes(widget)
	smallNote = tmpSmall
	if smallNote:
		karamba.resizeWidget(widget, size[0], 20)
		print "Init done"
	karamba.redrawWidget(widget)

#this is called everytime your widget is updated
#the update inverval is specified in the .theme file
def widgetUpdated(widget):

	global titleText, refreshRate, cmpt_refresh

	if titleType=='Date':
		date = time.localtime()
		titleText = time.strftime('%X', date)+'  '+ time.strftime('%x', date)
		karamba.changeText(widget, title, titleText+" ("+ str(currentPostIt+1)+'/'+str(len(myPostIts))+')')

	if cmpt_refresh>0:
		cmpt_refresh = cmpt_refresh-1
	else:
		cmpt_refresh = refreshRate
		reloadNotes(widget)
	karamba.redrawWidget(widget)

def reloadNotes(widget):
	global notesGatherer, skippedOnce
	global currentPostIt, myPostIts, title, smallNote, colors, xborder, yborder, titleType
	
	if smallNote:
		skippedOnce = True
		return
	skippedOnce = False
	
	notesTmp = notesGatherer.main()

	if not notesGatherer.needUpdate:
		print "Notes didn\'t change since last update"
		return

	print "Notes changed recently, update needed"
	for pi in myPostIts:
		pi.delete(widget)

	myPostIts = []
	myNotes = notesTmp
	vpos = 22
	p = PostIt()
	myPostIts.append(p)
	for n in myNotes:
		kn = karambaNote(widget, n, vpos)
		vpos = vpos + karamba.getRichTextSize(widget, kn.rt)[1]+3
		if vpos > size[1]-yborder and p.notes!=[]:
			vpos = 22
			kn.move(widget, vpos)
			vpos = vpos + karamba.getRichTextSize(widget, kn.rt)[1]+3
			p = PostIt()
			myPostIts.append(p)
		p.append(kn)
	if len(myPostIts)<=0:
		currentPostIt = 0
	elif len(myPostIts)<=currentPostIt:
		currentPostIt = len(myPostIts)-1
	updateCurrentPostIt(widget)
	notesGatherer.needUpdate = False
	
def updateCurrentPostIt(widget):
	global currentPostIt, myPostIts
	pi_index = 0
	for pi in myPostIts:
		pi.show(widget, pi_index==currentPostIt)
		pi_index = pi_index+1
	karamba.changeText(widget, title, titleText+" ("+ str(currentPostIt+1)+'/'+str(len(myPostIts))+')')
	karamba.changeTextColor(widget, title, int((colors[defaultColor])[0:3]), int((colors[defaultColor])[3:6]), int((colors[defaultColor])[6:9]))


#This gets called everytime our widget is clicked.
#Notes:
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button clicked:
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
#                    4,5 = Scroll wheel up and down
def widgetClicked(widget, x, y, button):
    pass

#This gets called everytime our widget is clicked.
#Notes
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button being held:
#                    0 = No Mouse Button
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
def widgetMouseMoved(widget, x, y, button):
    #Warning:  Don't do anything too intensive here
    #You don't want to run some complex piece of code everytime the mouse moves
    pass

#This gets called when an item is clicked in a popup menu you have created.
#  menu = a reference to the menu
#  id = the number of the item that was clicked.
def menuItemClicked(widget, menu, id):
	global select_img_pid, select_color_pid, select_title_pid, select_size_pid
	global configOptions, titleText, size, colors, xborder, yborder, select_ttBar_pid, bgImg
	global bgImgFile, ttBarImgFile, ttBarImg, refreshRate, select_refresh_pid
	global highlightOptions, hlCurrentNote
	if id==configOptions['title']:
		inputtext = karamba.readConfigEntry(widget, 'title')
		command = ["kdialog", '--caption', 'DesktopBasket',"--inputbox", _('Enter new title'), inputtext]
		select_title_pid = karamba.executeInteractive(widget, command)
	elif id==configOptions['color']:
		command = ["kdialog", '--caption', 'DesktopBasket',"--combobox", _('Select text default color')]
		cTrans=[]
		for c in colors.keys():
			cTrans.append( _(c) )
		cTrans.sort()
		command.extend(cTrans)
		select_color_pid = karamba.executeInteractive(widget, command)
	elif id==configOptions['size']:
		s = str(size[0])+' x '+str(size[1])
		command = ["kdialog", '--caption', 'DesktopBasket',"--inputbox", _('Enter new size'), s]
		select_size_pid = karamba.executeInteractive(widget, command)
	elif id==configOptions['background']:
		pth = karamba.getThemePath(widget)+"images/"
		command = ["kdialog", '--caption', 'DesktopBasket',"--getopenfilename", pth]
		select_img_pid = karamba.executeInteractive(widget, command)
	elif id==configOptions['titleBar']:
		pth = karamba.getThemePath(widget)+"images/"
		command = ["kdialog", '--caption', 'DesktopBasket', "--getopenfilename", pth]
		select_ttBar_pid = karamba.executeInteractive(widget, command)
	elif id==configOptions['selecttag']:
		askForDesiredTag(widget)
	elif id==configOptions['resetTheme']:
		bgImgFile = 'images/background.png'
		karamba.deleteImage(widget, bgImg)
		bgImg = karamba.createBackgroundImage(widget, 0, 20, bgImgFile)
		karamba.resizeImage(widget, bgImg, size[0], size[1]-20)
		karamba.writeConfigEntry(widget, "bgImg", bgImgFile)
		ttBarImgFile = 'images/TitleBar.png'
		karamba.deleteImage(widget, ttBarImg)
		ttBarImg = karamba.createBackgroundImage(widget, 0, 0, ttBarImgFile)
		karamba.resizeImage(widget, ttBarImg, size[0], 20)
		karamba.writeConfigEntry(widget, "titleBar", ttBarImgFile)
		karamba.redrawWidget(widget)
	elif id==configOptions['refresh']:
		notesGatherer.needUpdate=True
		reloadNotes(widget)
		karamba.redrawWidget(widget)
	elif id==configOptions['refreshRate']:
		command = ["kdialog", '--caption', 'DesktopBasket',"--inputbox", _('Enter new refresh rate (s)'), str(refreshRate)]
		select_refresh_pid = karamba.executeInteractive(widget, command)
	else:
		for color in highlightOptions.keys():
			if id==highlightOptions[color]:
				hlCurrentNote.changeHighlight(widget, color)
				break

def askForDesiredTag(widget):
	global select_tag_pid
	command = ['kdialog', '--caption', 'DesktopBasket', '--combobox', _('Select a tag') ]
	a = notesGatherer.choosingTags.keys()
	a.sort()
	command.extend(a)
	select_tag_pid = karamba.executeInteractive(widget, command)

#This gets called when an item is clicked in the theme CONFIGURATION menu,
#not the popup menus that you create.
#  key = the reference to the configuration key that was changed
#  value = the new value (true or false) that was selected
def menuOptionChanged(widget, key, value):
	global showTags, notesGatherer, titleType, title, titleText
	if key=='showTags':
		showTags= value==1
		notesGatherer.needUpdate=True
		karamba.writeConfigEntry(widget, "showTags", str(showTags))
	elif key=='titleDate':
		if value==1:
			titleType= 'Date'
		else:
			titleType= 'Text'
			titleText = karamba.readConfigEntry(widget, "title")
			if not titleText:
				titleText="DesktopBasket"
			updateCurrentPostIt(widget)
		karamba.writeConfigEntry(widget, "titleType", titleType)

#This gets called when a meter (image, text, etc) is clicked.
# NOTE you must use attachClickArea() to make a meter
# clickable.  
#  widget = reference to your theme
#  meter = the meter clicked
#  button = the button clicked (see widgetClicked for button numbers)
def meterClicked(widget, meter, button):
	global moveRightButton, moveLeftButton, myPostIts, currentPostIt, skippedOnce
	global launchBasketButton, cfgButton, smallNote, size, title, xborder, yborder
	global myPostIts, currentPostIt, highlightMenu, hlCurrentNote
	if button==1:	#left button
		if meter == moveLeftButton or meter == moveRightButton:
			if smallNote:
				return
			if meter == moveLeftButton and currentPostIt > 0 :
				currentPostIt = currentPostIt-1
			elif meter == moveRightButton and currentPostIt < len(myPostIts)-1 :
				currentPostIt = currentPostIt+1
			updateCurrentPostIt(widget)
		elif meter == launchBasketButton:
			print 'Running basket'
			if getoutput('dcop | grep basket'):
				karamba.execute('dcop basket MainWindow restore')
			else:
				karamba.execute('basket')
		elif meter == title:
			smallNote = not smallNote
			karamba.writeConfigEntry(widget, "isReduced", str(smallNote))
			if smallNote:
				karamba.resizeWidget(widget, size[0], 20)
			else:
				if skippedOnce:
					print "Reloading because the note was minimized for too long"
					reloadNotes(widget)
					karamba.redrawWidget(widget)
				karamba.resizeWidget(widget, size[0], size[1])
		elif meter == cfgButton:
			karamba.popupMenu(widget, configMenu, size[0]-100-xborder, 20 )
		else:
			for kn in myPostIts[currentPostIt].notes:
				if meter == kn.hlButton:
					pos = karamba.getImagePos(widget, kn.hlButton)
					hlCurrentNote = kn
					karamba.popupMenu(widget, highlightMenu, pos[0], pos[1]+18)

#This gets called when a command you have executed with executeInteractive() outputs something
#to stdout.  This way you can get the output of for example kdialog without freezing up the widget
#waiting for kdialog to end.
#  widget = reference to your theme
#  pid = process number of the program outputting (use this if you execute more than out process)
#  output = the text the program outputted to stdout
def commandOutput(widget, pid, output):
	global select_img_pid, select_color_pid, select_title_pid, select_size_pid
	global bgImgFile, bgImg, title, titleText, defaultColor, smallNote, xborder, yborder
	global ttBarImg, select_ttBar_pid, pref, desiredTag, select_tag_pid, notesGatherer, colors
	global select_refresh_pid, refreshRate, titleType, cmpt_refresh

	if output =='\n':
		return
	
	if pid == select_img_pid:
		bgImgFile = output
		karamba.deleteImage(widget, bgImg)
		bgImg = karamba.createBackgroundImage(widget, 0, 20, bgImgFile)
		karamba.resizeImage(widget, bgImg, size[0], size[1]-20)
		karamba.writeConfigEntry(widget, "bgImg", bgImgFile)
		karamba.redrawWidget(widget)

	elif pid == select_ttBar_pid:
		ttBarImgFile = output
		karamba.deleteImage(widget, ttBarImg)
		ttBarImg = karamba.createBackgroundImage(widget, 0, 0, ttBarImgFile)
		karamba.resizeImage(widget, ttBarImg, size[0], 20)
		karamba.writeConfigEntry(widget, "titleBar", ttBarImgFile)
		karamba.redrawWidget(widget)
		
	elif pid == select_title_pid:
		titleText = output[:-1]
		karamba.writeConfigEntry(widget, "title", titleText)
		try:
			titleText = unicode(titleText, 'utf-8')
		except:
			try:
				titleText = unicode(titleText, 'Latin1')
			except:
				pass
		karamba.changeText(widget, title, titleText+" ("+ str(currentPostIt+1)+'/'+str(len(myPostIts))+')')
		titleType= 'Text'
		karamba.writeConfigEntry(widget, "titleType", titleType)
		karamba.setMenuConfigOption(widget, 'titleDate', False)
		karamba.redrawWidget(widget)
		
	elif pid == select_color_pid:
		defaultColor = output[:-1]
		for c in colors.keys():
			if defaultColor==_(c):
				defaultColor=c
				break
		karamba.writeConfigEntry(widget, "defaultColor", defaultColor)
		tmpSmall = smallNote
		smallNote = False
		notesGatherer.needUpdate=True
		reloadNotes(widget)
		karamba.redrawWidget(widget)
		smallNote = tmpSmall
		
	elif pid == select_size_pid:
		s = output[:-1].lower()
		if s.find('x') == -1:
			return
		ss = s.split('x')
		size[0]= int(ss[0])
		size[1]= int(ss[1])
		karamba.resizeWidget(widget, size[0], size[1])
		karamba.resizeImage(widget, bgImg, size[0], size[1]-20)
		karamba.resizeImage(widget, ttBarImg, size[0], 20)
		karamba.moveImage(widget, launchBasketButton, size[0]-16-xborder, 2)
		karamba.moveImage(widget, cfgButton, size[0]-36-xborder, 2)
		karamba.resizeText(widget, title, size[0]-100-2*xborder, 20)
		karamba.writeConfigEntry(widget, "noteHeight", str(size[1]))
		notesGatherer.needUpdate=True
		karamba.redrawWidget(widget)
		smallNote = tmpSmall

	elif pid == select_tag_pid:
		tmp = output[:-1]
		try:
			tmp = unicode(tmp, 'utf8')
		except:
			print "Error while decoding note(utf8)"
			try:
				tmp = unicode(tmp, 'Latin1')
			except:
				print "Error while decoding note(Latin1)"
		desiredTag = notesGatherer.choosingTags[tmp]
		karamba.writeConfigEntry(widget, "desiredTag", desiredTag)
		print "New preferred tag is", desiredTag
		cmpt_refresh = 1
		karamba.redrawWidget(widget)

	elif pid == select_refresh_pid:
		refreshRate = int(output[:-1].lower())
		karamba.writeConfigEntry(widget, "refreshRate", str(refreshRate))

#This gets called when another theme sends a message.
#  widget = reference to your theme
#  caller = "pretty" name or regular name (name of theme file with a number attached) of sending theme
#  output = the text sent.
def themeNotify(widget, caller, message):
    pass

#This gets called when an item is dropped on this widget.
# NOTE you have to call acceptDrops() before your widget will accept drops.
#  widget = reference to your theme
#  dropText = the text of the dropped item (probably a URL to it's location in KDE)
#  x = the x position on the theme relative to the upper left corner
#  y = the y position on the theme relative to the upper left corner
def itemDropped(widget, dropText, x, y):
    pass


#This gets called when a new program is LOADING in KDE.  When it is done
#loading, startupRemoved() is called, followed by taskAdded().
#  widget = reference to your widget
#  task = A refence to the task that is starting.  
def startupAdded(widget, startup):
    pass

#This gets called when a new program is done LOADING in KDE.
#  widget = reference to your widget
#  task = A refence to the task that just finished loading.  
def startupRemoved(widget, startup):
    pass

#This is called every time a new task (program) is started in KDE.
#  widget = reference to your widget
#  task = A refence to the new task.  Call getTaskInfo() with this reference
#         to get the name, etc of this new task.
def taskAdded(widget, task):
    pass

#This is called everytime a task (program) is closed in KDE.
#  widget = reference to your widget
#  task = A refence to the task.  
def taskRemoved(widget, task):
    pass

#This is called everytime a different task gains focus (IE, the user clicks
#on a different window).  
#  widget = reference to your widget
#  task = A refence to the task.  Call getTaskInfo() with this reference
#         to get the name, etc of this new task.
def activeTaskChanged(widget, task):
    pass

#This is called everytime the systray you created with createSystray() is updated
def systrayUpdated(widget):
    pass

#This is called everytime the current desktop changes
#  widget = reference to your widget
#  desktop = the current desktop
def desktopChanged(widget, desktop):
    pass

#This is called everytime the wallpaper changes on a desktop 
#  widget = reference to your widget
#  desktop = the desktop whose wallpaper changed
def wallpaperChanged(widget, desktop):
    pass

#This is called everytime there is a key press in any focused input field
#  widget = reference to your widget
#  meter = reference to an input box
#  char = the key that was pressed
def keyPressed(widget, meter, char):
    pass

def widgetClosed(widget):
    pass

# This will be printed when the widget loads.
print "Loaded DesktopBasket"
